//------------------------------------------------------------------------------
// <copyright company="Kartones.Net">
//     Copyleft (c) Kartones 2009
// </copyright> 
//------------------------------------------------------------------------------

using System;
using System.Collections;
using CommunityServer.Components;
using CommunityServer.Blogs.Components;
using CommunityServer.Galleries.Components;
using CommunityServer.Files.Components;
using CommunityServer.Spam;
using CommunityServer.Spam.Rules;

namespace KartonesNet.Modules.Rules
{
    /// <summary>
    /// Rates spam factor based on number of occurences of configured bad words. 
    /// Uses substring searches instead of regular expressions.
    /// </summary>
    /// <remarks>Based on CS2007 BadWordSpamRule</remarks>
    public class BadWordNoRegExpSpamRule : SpamRule
    {
        #region Fields

        private static readonly Guid ruleID = new Guid("660CF269-9824-43CA-A7A1-D7A1E952CFFA");
        private static readonly int defaultPoints = 2;
        private static readonly int defaultThreshold = 1;
        private static readonly string defaultBadWords = 
            "-online;4u;adipex;advicer;ambien;art works;baccarrat;blackjack;bllogspot;booker;byob;car-rental-;carisoprodol;cartwish;casino;chatroom;china;cialis;coolhu;credit-;cwas;cyclen;cyclobenzaprine;dating-;day-trading;debt-consolidation;download;drug;discreetordering;duty-free;dutyfree;edit1;equityloans;fioricet;flowers-;free;freenet;gambling;health-insurancedeals-;holdem;homeequityloans;homefinance;hotel-deals;hotele-site;hotelse-site;incest;insurance-quotesdeals-;insurancedeals-;jrcreations;levitra;loan;macinstruct;mortgage-4;mortgagequotes;online-gambling;onlinegambling-;ottawavalleyag;ownsthis;paxil;pen1s;penis;pharmacy;phentermine;pills;poker;porn;poze;rental-car-;right-top;ripoffs;roulette;search-browse;sex;shemale;shoes;slot;soma;sysworkds;taboo;teen;texas;thorcarlson;top-site;top-e-;tramadol;trim-spa;ultram;valeofglamorganconservatives;viagra;vioxx;xanax;zolus";
        private static readonly string resourceFile = "ControlPanelResources.xml";

        #endregion

        #region Properties

        /// <summary>
        /// Name of the Spam Rule
        /// </summary>
        public override string Name
        {
            get
            {
                return "Bad Word Count (No RegExp)";
            }
        }

        /// <summary>
        /// Description of the Spam rule
        /// </summary>
        public override string Description
        {
            get
            {
                return "Rates spam factor based on number of occurences of configured bad words, without using Regular Expressions";
            }
        }

        /// <summary>
        /// GUID of the Spam Rule
        /// </summary>
        public override Guid RuleID
        {
            get
            {
                return ruleID;
            }
        }

        #endregion

        #region Class Constructor

        /// <summary>
        /// Class Constructor
        /// </summary>
        public BadWordNoRegExpSpamRule()
		{
        }

        #endregion

        #region SpamRule Methods

        /// <summary>
        /// Gets the Spam Rule settings
        /// </summary>
        /// <returns>Arraylist with the settings</returns>
        public override ArrayList GetAvailableSettings()
        {
            ArrayList settings = new ArrayList();
            settings.Add(new RuleSetting(ruleID, "points", 
                ResourceManager.GetString("spam_WordCountRule_PointsDescription", resourceFile), 
                defaultPoints.ToString()));
            settings.Add(new RuleSetting(ruleID, "threshold", 
                ResourceManager.GetString("spam_WordCountRule_ThresholdDescription", resourceFile), 
                defaultThreshold.ToString()));
            settings.Add(new RuleSetting(ruleID, "badwords", 
                ResourceManager.GetString("spam_WordCountRule_BadWordsDescription", resourceFile), 
                defaultBadWords, 8000));
            return settings;
        }

        /// <summary>
        /// Rates a post/comment
        /// </summary>
        /// <param name="Post">Post to rate</param>
        /// <param name="Args">CSPostEventArgs</param>
        /// <returns>Post spam rating</returns>
        public override int RatePost(Post Post, CSPostEventArgs Args)
        {
            int counter;
            int points;
            int threshold;
            string badWordList;
            string[] badWords;

            // Verify that this spam rule should process this post
            // Also, only apply to anonymous users
            if (IsExemptFromSpamRating(Post, Args) || (!Post.User.IsAnonymous))
            {
                return 0;
            }

            // Retrieve setting values
            points = Globals.SafeInt(this.GetSettingValue("points"), defaultPoints);
            threshold = Globals.SafeInt(this.GetSettingValue("threshold"), defaultThreshold);
            badWordList = GetSettingValue("badwords");
            if (badWordList == null)
            {
                badWordList = defaultBadWords;
            }
            if (badWordList.Trim().Length <= 0)
            {
                return 0;
            }
            badWords = badWordList.Split(';');

            // Calc. spam value
            if (!Globals.IsNullorEmpty(Post.Body))
            {
                if (badWords == null || badWords.Length == 0)
                {
                    return 0;
                }

                try
                {
                    counter = 0;
                    // For faster processing, assuming no whitespaces and empty ";"
                    for (int index = 0; index < badWords.Length; index++)
                    {
                        if (badWords[index].Length > 0)
                        {
                            counter += CountBadWordOccurences(badWords[index], Post);
                        }
                    }

                    // Score xxx points for each badword above the threshold
                    if (counter > threshold)
                        return (points * (counter - threshold));
                }
                catch (Exception ex)
                {
                    EventLogs.Warn(
                    string.Format("Spam Rule exception: {0}", ex.Message), "BadWordNoRegExpSpamRule", 982);
                }
            }

            return 0;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Counts the number of occurences of a bad word in a post.
        /// </summary>
        /// <param name="Word">Word to search for</param>
        /// <param name="Post">Post to search into</param>
        /// <returns>Times it appears on the post</returns>
        private int CountBadWordOccurences(string Word, Post Post)
        {
            int count = 0;

            count = DoCountBadWord(Word, Post.Body);
            count += DoCountBadWord(Word, Post.Subject);

            // Check for Blog specific properties
            WeblogPost weblogPost = Post as WeblogPost;
            if (weblogPost != null)
            {
                if (weblogPost.HasSubmittedUserName)
                {
                    count += DoCountBadWord(Word, weblogPost.SubmittedUserName);
                }
                if (!Globals.IsNullorEmpty(weblogPost.TrackBackName))
                {
                    count += DoCountBadWord(Word, weblogPost.TrackBackName);
                }
                if (weblogPost.HasTitleUrl)
                {
                    count += DoCountBadWord(Word, weblogPost.TitleUrl);
                }
            }

            // Check for Photo Gallery specific properties
            GalleryPost galleryPost = Post as GalleryPost;
            if (galleryPost != null)
            {
                if (galleryPost.HasSubmittedUserName)
                {
                    count += DoCountBadWord(Word, galleryPost.SubmittedUserName);
                }
                if (!Globals.IsNullorEmpty(galleryPost.TrackBackName))
                {
                    count += DoCountBadWord(Word, galleryPost.TrackBackName);
                }
                if (galleryPost.HasTitleUrl)
                {
                    count += DoCountBadWord(Word, galleryPost.TitleUrl);
                }
            }

            // Check for File Gallery specific properties
            EntryComment entryComment = Post as EntryComment;
            if (entryComment != null)
            {
                if (entryComment.HasSubmittedUserName)
                {
                    count += DoCountBadWord(Word, entryComment.SubmittedUserName);
                }
                if (entryComment.HasTitleUrl)
                {
                    count += DoCountBadWord(Word, entryComment.TitleUrl);
                }
            }

            return count;
        }

        /// <summary>
        /// Counts the number of occurences of a bad word in a string.
        /// </summary>
        /// <param name="Word">Word to search for</param>
        /// <param name="Text">Post to search into</param>
        /// <returns>Times it appears on the text</returns>
        private int DoCountBadWord(string Word, string Text)
        {
            int count = 0;
            int position = 0;
            int lastPosition = -1;

            lastPosition = Text.LastIndexOf(Word, StringComparison.OrdinalIgnoreCase);
            if (lastPosition == -1)
            {
                return 0;
            }

            while (position >= 0)
            {
                position = Text.IndexOf(Word, position, StringComparison.OrdinalIgnoreCase);
                if (position >= 0)
                {
                    count++;
                }
                if (position == lastPosition)
                {
                    break;
                }
            }

            return count;
        }

        #endregion
    }
}
